@@ -545,7 +545,8 @@ if __name__ == '__main__':
     logger.init()
     print version.BANNER
 
-    parser = argparse.ArgumentParser(add_help = True, description = "Test whether an account is valid on the target host using the RDP protocol.")
+    parser = argparse.ArgumentParser(add_help = True, description = "Test whether an account is valid on the target "
+                                                                    "host using the RDP protocol.")
 
     parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
 
