@@ -28,16 +28,12 @@ from impacket.crypto import *
 
 
 class SVCCTL:
-    KNOWN_PROTOCOLS = {
-        '139/SMB': (r'ncacn_np:%s[\pipe\svcctl]', 139),
-        '445/SMB': (r'ncacn_np:%s[\pipe\svcctl]', 445),
-        }
 
-    def __init__(self, username, password, domain, options):
+    def __init__(self, username, password, domain, options, port=445):
         self.__username = username
         self.__password = password
-        self.__protocol = SVCCTL.KNOWN_PROTOCOLS.keys()
         self.__options = options
+        self.__port = port
         self.__action = options.action.upper()
         self.__domain = domain
         self.__lmhash = ''
@@ -49,34 +45,19 @@ class SVCCTL:
         if options.hashes is not None:
             self.__lmhash, self.__nthash = options.hashes.split(':')
 
-    def run(self, addr):
+    def run(self, remoteName, remoteHost):
 
-        # Try all requested protocols until one works.
-        for protocol in self.__protocol:
-            protodef = SVCCTL.KNOWN_PROTOCOLS[protocol]
-            port = protodef[1]
+        stringbinding = 'ncacn_np:%s[\pipe\svcctl]' % remoteName
+        logging.debug('StringBinding %s'%stringbinding)
+        rpctransport = transport.DCERPCTransportFactory(stringbinding)
+        rpctransport.set_dport(self.__port)
+        rpctransport.setRemoteHost(remoteHost)
+        if hasattr(rpctransport, 'set_credentials'):
+            # This method exists only for selected protocol sequences.
+            rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash, self.__aesKey)
 
-            logging.info("Trying protocol %s..." % protocol)
-            stringbinding = protodef[0] % addr
-
-            rpctransport = transport.DCERPCTransportFactory(stringbinding)
-            rpctransport.set_dport(port)
-            rpctransport.set_kerberos(self.__doKerberos, self.__kdcHost)
-            if hasattr(rpctransport, 'set_credentials'):
-                # This method exists only for selected protocol sequences.
-                rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash,
-                                             self.__nthash, self.__aesKey)
-
-            try:
-                self.doStuff(rpctransport)
-            except Exception, e:
-                #import traceback
-                #traceback.print_exc()
-                logging.critical(str(e))
-                break
-            else:
-                # Got a response. No need for further iterations.
-                break
+        rpctransport.set_kerberos(self.__doKerberos, self.__kdcHost)
+        self.doStuff(rpctransport)
 
     def doStuff(self, rpctransport):
         dce = rpctransport.get_dce_rpc()
@@ -133,7 +114,7 @@ class SVCCTL:
             elif resp['lpServiceConfig']['dwStartType'] == 0x4:
                 print "DISABLED"
             else:
-                print "UNKOWN"
+                print "UNKNOWN"
 
             print "ERROR_CONTROL     : %2d - " % resp['lpServiceConfig']['dwErrorControl'],
             if resp['lpServiceConfig']['dwErrorControl'] == 0x0:
@@ -145,7 +126,7 @@ class SVCCTL:
             elif resp['lpServiceConfig']['dwErrorControl'] == 0x3:
                 print "CRITICAL"
             else:
-                print "UNKOWN"
+                print "UNKNOWN"
             print "BINARY_PATH_NAME  : %s" % resp['lpServiceConfig']['lpBinaryPathName'][:-1]
             print "LOAD_ORDER_GROUP  : %s" % resp['lpServiceConfig']['lpLoadOrderGroup'][:-1]
             print "TAG               : %d" % resp['lpServiceConfig']['dwTagId']
@@ -172,7 +153,7 @@ class SVCCTL:
             elif state == scmr.SERVICE_STOPPED:
                print "STOPPED"
             else:
-               print "UNKOWN"
+               print "UNKNOWN"
         elif self.__action == 'LIST':
             logging.info("Listing services available on target")
             #resp = rpc.EnumServicesStatusW(scManagerHandle, svcctl.SERVICE_WIN32_SHARE_PROCESS )
@@ -197,7 +178,7 @@ class SVCCTL:
                 elif state == scmr.SERVICE_STOPPED:
                    print "STOPPED"
                 else:
-                   print "UNKOWN"
+                   print "UNKNOWN"
             print "Total Services: %d" % len(resp)
         elif self.__action == 'CREATE':
             logging.info("Creating service %s" % self.__options.name)
@@ -309,16 +290,30 @@ if __name__ == '__main__':
     create_parser.add_argument('-path', action='store', required=False, help='binary path')
     create_parser.add_argument('-service_type', action='store', required=False, help='service type')
     create_parser.add_argument('-start_type', action='store', required=False, help='service start type')
-    create_parser.add_argument('-start_name', action='store', required=False, help='string that specifies the name of the account under which the service should run')
-    create_parser.add_argument('-password', action='store', required=False, help='string that contains the password of the account whose name was specified by the start_name parameter')
+    create_parser.add_argument('-start_name', action='store', required=False, help='string that specifies the name of '
+                               'the account under which the service should run')
+    create_parser.add_argument('-password', action='store', required=False, help='string that contains the password of '
+                               'the account whose name was specified by the start_name parameter')
 
     group = parser.add_argument_group('authentication')
 
     group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
     group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
-    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
-    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
-    group.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller. If ommited it use the domain part (FQDN) specified in the target parameter')
+    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file '
+                       '(KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the '
+                       'ones specified in the command line')
+    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication '
+                                                                            '(128 or 256 bits)')
+
+    group = parser.add_argument_group('connection')
+
+    group.add_argument('-dc-ip', action='store',metavar = "ip address", help='IP Address of the domain controller. If '
+                       'ommited it use the domain part (FQDN) specified in the target parameter')
+    group.add_argument('-target-ip', action='store', metavar="ip address", help='IP Address of the target machine. If '
+                       'ommited it will use whatever was specified as target. This is useful when target is the NetBIOS '
+                       'name and you cannot resolve it')
+    group.add_argument('-port', choices=['139', '445'], nargs='?', default='445', metavar="destination port",
+                       help='Destination port to connect to SMB Server')
  
     if len(sys.argv)==1:
         parser.print_help()
@@ -333,17 +328,20 @@ if __name__ == '__main__':
 
     import re
 
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
+    domain, username, password, remoteName = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
         options.target).groups('')
 
     #In case the password contains '@'
-    if '@' in address:
-        password = password + '@' + address.rpartition('@')[0]
-        address = address.rpartition('@')[2]
+    if '@' in remoteName:
+        password = password + '@' + remoteName.rpartition('@')[0]
+        remoteName = remoteName.rpartition('@')[2]
 
     if domain is None:
         domain = ''
 
+    if options.target_ip is None:
+        options.target_ip = remoteName
+
     if options.aesKey is not None:
         options.k = True
 
@@ -351,8 +349,11 @@ if __name__ == '__main__':
         from getpass import getpass
         password = getpass("Password:")
 
-    services = SVCCTL(username, password, domain, options)
+    services = SVCCTL(username, password, domain, options, int(options.port))
     try:
-        services.run(address)
+        services.run(remoteName, options.target_ip)
     except Exception, e:
+        if logging.getLogger().level == logging.DEBUG:
+            import traceback
+            traceback.print_exc()
         logging.error(str(e))
