@@ -36,6 +36,7 @@ from impacket.dcerpc.v5.dcom import wmi
 from impacket.dcerpc.v5.dtypes import NULL
 
 OUTPUT_FILENAME = '__' + str(time.time())
+CODEC = sys.stdout.encoding
 
 class WMIEXEC:
     def __init__(self, command='', username='', password='', domain='', hashes=None, aesKey=None, share=None,
@@ -92,8 +93,9 @@ class WMIEXEC:
             else:
                 self.shell.cmdloop()
         except  (Exception, KeyboardInterrupt), e:
-            #import traceback
-            #traceback.print_exc()
+            if logging.getLogger().level == logging.DEBUG:
+                import traceback
+                traceback.print_exc()
             logging.error(str(e))
             if smbConnection is not None:
                 smbConnection.logoff()
@@ -109,12 +111,12 @@ class RemoteShell(cmd.Cmd):
     def __init__(self, share, win32Process, smbConnection):
         cmd.Cmd.__init__(self)
         self.__share = share
-        self.__output = '\\' + OUTPUT_FILENAME 
-        self.__outputBuffer = ''
+        self.__output = '\\' + OUTPUT_FILENAME
+        self.__outputBuffer = unicode('')
         self.__shell = 'cmd.exe /Q /c '
         self.__win32Process = win32Process
         self.__transferClient = smbConnection
-        self.__pwd = 'C:\\'
+        self.__pwd = unicode('C:\\')
         self.__noOutput = False
         self.intro = '[!] Launching semi-interactive shell - Careful what you execute\n[!] Press help for extra shell commands'
 
@@ -147,6 +149,7 @@ class RemoteShell(cmd.Cmd):
                 logging.error(str(e))
 
     def do_get(self, src_path):
+
         try:
             import ntpath
             newPath = ntpath.normpath(ntpath.join(self.__pwd, src_path))
@@ -156,10 +159,14 @@ class RemoteShell(cmd.Cmd):
             logging.info("Downloading %s\\%s" % (drive, tail))
             self.__transferClient.getFile(drive[:-1]+'$', tail, fh.write)
             fh.close()
+
         except Exception, e:
             logging.error(str(e))
-            os.remove(filename)
-            pass
+
+            if os.path.exists(filename):
+                os.remove(filename)
+
+
 
     def do_put(self, s):
         try:
@@ -194,13 +201,13 @@ class RemoteShell(cmd.Cmd):
         self.execute_remote('cd ' + s)
         if len(self.__outputBuffer.strip('\r\n')) > 0:
             print self.__outputBuffer
-            self.__outputBuffer = ''
+            self.__outputBuffer = u''
         else:
-            self.__pwd = ntpath.normpath(ntpath.join(self.__pwd, s))
+            self.__pwd = ntpath.normpath(ntpath.join(self.__pwd, s.decode(sys.stdin.encoding)))
             self.execute_remote('cd ')
             self.__pwd = self.__outputBuffer.strip('\r\n')
-            self.prompt = self.__pwd + '>'
-            self.__outputBuffer = ''
+            self.prompt = unicode(self.__pwd + '>').encode(CODEC)
+            self.__outputBuffer = u''
 
     def default(self, line):
         # Let's try to guess if the user is trying to change drive
@@ -210,24 +217,30 @@ class RemoteShell(cmd.Cmd):
             if len(self.__outputBuffer.strip('\r\n')) > 0: 
                 # Something went wrong
                 print self.__outputBuffer
-                self.__outputBuffer = ''
+                self.__outputBuffer = u''
             else:
                 # Drive valid, now we should get the current path
                 self.__pwd = line
                 self.execute_remote('cd ')
                 self.__pwd = self.__outputBuffer.strip('\r\n')
-                self.prompt = self.__pwd + '>'
-                self.__outputBuffer = ''
+                self.prompt = unicode(self.__pwd + '>').encode(CODEC)
+                self.__outputBuffer = u''
         else:
             if line != '':
                 self.send_data(line)
 
     def get_output(self):
         def output_callback(data):
-            self.__outputBuffer += data
+            try:
+                self.__outputBuffer += data.decode(CODEC)
+            except UnicodeDecodeError, e:
+                logging.error('Decoding error detected, consider running chcp.com at the target,\nmap the result with '
+                              'https://docs.python.org/2.4/lib/standard-encodings.html\nand then execute wmiexec.py '
+                              'again with -codec and the corresponding codec')
+                self.__outputBuffer += data.decode(CODEC, errors='replace')
 
         if self.__noOutput is True:
-            self.__outputBuffer = ''
+            self.__outputBuffer = u''
             return
 
         while True:
@@ -239,23 +252,73 @@ class RemoteShell(cmd.Cmd):
                     # Output not finished, let's wait
                     time.sleep(1)
                     pass
-                else:
-                    #print str(e)
-                    pass 
+                elif str(e).find('Broken') >= 0:
+                    # The SMB Connection might have timed out, let's try reconnecting
+                    logging.debug('Connection broken, trying to recreate it')
+                    self.__transferClient.reconnect()
+                    return self.get_output()
         self.__transferClient.deleteFile(self.__share, self.__output)
 
     def execute_remote(self, data):
         command = self.__shell + data 
         if self.__noOutput is False:
             command += ' 1> ' + '\\\\127.0.0.1\\%s' % self.__share + self.__output  + ' 2>&1'
-        self.__win32Process.Create(command, self.__pwd, None)
+        self.__win32Process.Create(command.decode(sys.stdin.encoding), self.__pwd, None)
         self.get_output()
 
     def send_data(self, data):
         self.execute_remote(data)
         print self.__outputBuffer
-        self.__outputBuffer = ''
+        self.__outputBuffer = u''
 
+class AuthFileSyntaxError(Exception):
+    
+    '''raised by load_smbclient_auth_file if it encounters a syntax error
+    while loading the smbclient-style authentication file.'''
+
+    def __init__(self, path, lineno, reason):
+        self.path=path
+        self.lineno=lineno
+        self.reason=reason
+    
+    def __str__(self):
+        return 'Syntax error in auth file %s line %d: %s' % (
+            self.path, self.lineno, self.reason )
+
+def load_smbclient_auth_file(path):
+
+    '''Load credentials from an smbclient-style authentication file (used by
+    smbclient, mount.cifs and others).  returns (domain, username, password)
+    or raises AuthFileSyntaxError or any I/O exceptions.'''
+
+    lineno=0
+    domain=None
+    username=None
+    password=None
+    for line in open(path):
+        lineno+=1
+
+        line = line.strip()
+
+        if line.startswith('#') or line=='':
+            continue
+            
+        parts = line.split('=',1)
+        if len(parts) != 2:
+            raise AuthFileSyntaxError(path, lineno, 'No "=" present in line')
+        
+        (k,v) = (parts[0].strip(), parts[1].strip())
+        
+        if k=='username':
+            username=v
+        elif k=='password':
+            password=v
+        elif k=='domain':
+            domain=v
+        else:
+            raise AuthFileSyntaxError(path, lineno, 'Unknown option %s' % repr(k))
+            
+    return (domain, username, password)
 
 # Process command-line arguments.
 if __name__ == '__main__':
@@ -263,21 +326,36 @@ if __name__ == '__main__':
     logger.init()
     print version.BANNER
 
-    parser = argparse.ArgumentParser(add_help = True, description = "Executes a semi-interactive shell using Windows Management Instrumentation.")
+    parser = argparse.ArgumentParser(add_help = True, description = "Executes a semi-interactive shell using Windows "
+                                                                    "Management Instrumentation.")
     parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
-    parser.add_argument('-share', action='store', default = 'ADMIN$', help='share where the output will be grabbed from (default ADMIN$)')
-    parser.add_argument('-nooutput', action='store_true', default = False, help='whether or not to print the output (no SMB connection created)')
+    parser.add_argument('-share', action='store', default = 'ADMIN$', help='share where the output will be grabbed from '
+                                                                           '(default ADMIN$)')
+    parser.add_argument('-nooutput', action='store_true', default = False, help='whether or not to print the output '
+                                                                                '(no SMB connection created)')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
+    parser.add_argument('-codec', action='store', help='Sets encoding used (codec) from the target\'s output (default '
+                                                       '"%s"). If errors are detected, run chcp.com at the target, '
+                                                       'map the result with '
+                          'https://docs.python.org/2.4/lib/standard-encodings.html and then execute wmiexec.py '
+                          'again with -codec and the corresponding codec ' % CODEC)
 
-    parser.add_argument('command', nargs='*', default = ' ', help='command to execute at the target. If empty it will launch a semi-interactive shell')
+    parser.add_argument('command', nargs='*', default = ' ', help='command to execute at the target. If empty it will '
+                                                                  'launch a semi-interactive shell')
 
     group = parser.add_argument_group('authentication')
 
     group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
     group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
-    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file (KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the ones specified in the command line')
-    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication (128 or 256 bits)')
-    group.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller. If ommited it use the domain part (FQDN) specified in the target parameter')
+    group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file '
+                       '(KRB5CCNAME) based on target parameters. If valid credentials cannot be found, it will use the '
+                       'ones specified in the command line')
+    group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication '
+                                                                            '(128 or 256 bits)')
+    group.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller. If '
+                       'ommited it use the domain part (FQDN) specified in the target parameter')
+    group.add_argument('-A', action="store", metavar = "authfile", help="smbclient/mount.cifs-style authentication file. "
+                                                                        "See smbclient man page's -A option.")
 
     if len(sys.argv)==1:
         parser.print_help()
@@ -285,6 +363,12 @@ if __name__ == '__main__':
 
     options = parser.parse_args()
 
+    if options.codec is not None:
+        CODEC = options.codec
+    else:
+        if CODEC is None:
+            CODEC = 'UTF-8'
+
     if ' '.join(options.command) == ' ' and options.nooutput is True:
         logging.error("-nooutput switch and interactive shell not supported")
         sys.exit(1)
@@ -305,6 +389,10 @@ if __name__ == '__main__':
         address = address.rpartition('@')[2]
 
     try:
+        if options.A is not None:
+            (domain, username, password) = load_smbclient_auth_file(options.A)
+            logging.debug('loaded smbclient auth file: domain=%s, username=%s, password=%s' % (repr(domain), repr(username), repr(password)))
+        
         if domain is None:
             domain = ''
 
@@ -318,8 +406,13 @@ if __name__ == '__main__':
         executer = WMIEXEC(' '.join(options.command), username, password, domain, options.hashes, options.aesKey,
                            options.share, options.nooutput, options.k, options.dc_ip)
         executer.run(address)
-    except (Exception, KeyboardInterrupt), e:
-        #import traceback
-        #print traceback.print_exc()
+    except KeyboardInterrupt, e:
         logging.error(str(e))
+    except Exception, e:
+        if logging.getLogger().level == logging.DEBUG:
+            import traceback
+            traceback.print_exc()
+        logging.error(str(e))
+        sys.exit(1)
+        
     sys.exit(0)
