@@ -16,6 +16,8 @@
 #   [MS-LSAD] Section 5.1.2
 #   [MS-SAMR] Section 2.2.11.1.1
 
+from __future__ import division
+from __future__ import print_function
 from impacket import LOG
 try:
     from Crypto.Cipher import DES, AES, ARC4
@@ -141,7 +143,7 @@ def AES_CMAC(K, M, length):
     AES_128= AES.new(K)
     M      = M[:length]
     K1, K2 = Generate_Subkey(K)
-    n      = len(M)/const_Bsize
+    n      = len(M)//const_Bsize
 
     if n == 0:
         n = 1
@@ -219,13 +221,13 @@ def KDF_CounterMode(KI, Label, Context, L):
     h = 256
     r = 32
     
-    n = L / h
+    n = L // h
 
     if n == 0:
         n = 1
 
     if n > (pow(2,r)-1):
-        raise "Error computing KDF_CounterMode"
+        raise Exception("Error computing KDF_CounterMode")
 
     result = ''
     K      = ''
@@ -235,7 +237,7 @@ def KDF_CounterMode(KI, Label, Context, L):
        K = hmac.new(KI, input, hashlib.sha256).digest() 
        result = result + K
  
-    return result[:(L/8)]      
+    return result[:(L//8)]
 
 # [MS-LSAD] Section 5.1.2 / 5.1.3
 class LSA_SECRET_XP(Structure):
@@ -379,11 +381,13 @@ if __name__ == '__main__':
 #                  f69f2445 df4f9b17 ad2b417b e66c3710
 #   AES-CMAC       51f0bebf 7e3b9d92 fc497417 79363cfe
 #  --------------------------------------------------
-    def pp(s):
-        for i in range((len(s)/8)):
-            print s[:8] ,
+    def pp(prev ,s):
+        print (prev, end= ' ')
+        for i in range((len(s)//8)):
+            print("%s" % (s[:8]), end = ' ')
             s = s[8:]
 
+        print()
         return ''
     
     from binascii import hexlify, unhexlify
@@ -392,33 +396,33 @@ if __name__ == '__main__':
     M = "6bc1bee22e409f96e93d7e117393172aae2d8a571e03ac9c9eb76fac45af8e5130c81c46a35ce411e5fbc1191a0a52eff69f2445df4f9b17ad2b417be66c3710"
 
     K1, K2 = Generate_Subkey(unhexlify(K))
-    print "Subkey Generation"
-    print "K               ", pp(K)
-    print "K1              ", pp(hexlify(K1))
-    print "K2              ", pp(hexlify(K2))
-    print 
-    print "Example 1: len = 0"
-    print "M                <empty string>"
-    print "AES-CMAC        " , pp(hexlify(AES_CMAC(unhexlify(K),unhexlify(M),0)))
-    print 
-    print "Example 2: len = 16"
-    print "M               " , pp(M[:16*2])
-    print "AES-CMAC        " , pp(hexlify(AES_CMAC(unhexlify(K),unhexlify(M),16)))
-    print 
-    print "Example 3: len = 40"
-    print "M               " , pp(M[:40*2])
-    print "AES-CMAC        " , pp(hexlify(AES_CMAC(unhexlify(K),unhexlify(M),40)))
-    print 
-    print "Example 3: len = 64"
-    print "M               " , pp(M[:64*2])
-    print "AES-CMAC        " , pp(hexlify(AES_CMAC(unhexlify(K),unhexlify(M),64)))
-    print 
+    print("Subkey Generation")
+    pp("K               ", (K))
+    pp("K1              ", (hexlify(K1)))
+    pp("K2              ", (hexlify(K2)))
+    print()
+    print("Example 1: len = 0")
+    print("M                <empty string>")
+    pp("AES-CMAC        " , (hexlify(AES_CMAC(unhexlify(K),unhexlify(M),0))))
+    print() 
+    print("Example 2: len = 16")
+    pp("M               " , (M[:16*2]))
+    pp("AES-CMAC        " , (hexlify(AES_CMAC(unhexlify(K),unhexlify(M),16))))
+    print() 
+    print("Example 3: len = 40")
+    pp("M               " , (M[:40*2]))
+    pp("AES-CMAC        " , (hexlify(AES_CMAC(unhexlify(K),unhexlify(M),40))))
+    print() 
+    print("Example 3: len = 64")
+    pp("M               " , (M[:64*2]))
+    pp("AES-CMAC        " , (hexlify(AES_CMAC(unhexlify(K),unhexlify(M),64))))
+    print() 
     M = "eeab9ac8fb19cb012849536168b5d6c7a5e6c5b2fcdc32bc29b0e3654078a5129f6be2562046766f93eebf146b"
     K = "6c3473624099e17ff3a39ff6bdf6cc38"
     # Mac = dbf63fd93c4296609e2d66bf79251cb5
-    print "Example 4: len = 45"
-    print "M               " , pp(M[:45*2])
-    print "AES-CMAC        " , pp(hexlify(AES_CMAC(unhexlify(K),unhexlify(M),45)))
+    print("Example 4: len = 45")
+    pp("M               " , (M[:45*2]))
+    pp("AES-CMAC        " , (hexlify(AES_CMAC(unhexlify(K),unhexlify(M),45))))
 
 #   ------------------------------------------------------------
 #
@@ -445,23 +449,23 @@ if __name__ == '__main__':
     K = "000102030405060708090a0b0c0d0e0fedcb"
     M = "000102030405060708090a0b0c0d0e0f10111213"
 
-    print "AES-CMAC-PRF-128 Test Vectors"
-    print
-    print "Example 1: len = 0"
-    print "M               " , pp(K)
-    print "Key Length       18 "  
-    print "AES-CMAC        " , pp(hexlify(AES_CMAC_PRF_128(unhexlify(K),unhexlify(M),18,len(unhexlify(M)))))
-    print 
-    print "Example 1: len = 0"
-    print "M               " , pp(K)
-    print "Key Length       16 "  
-    print "AES-CMAC        " , pp(hexlify(AES_CMAC_PRF_128(unhexlify(K)[:16],unhexlify(M),16,len(unhexlify(M)))))
-    print 
-    print "Example 1: len = 0"
-    print "M               " , pp(K)
-    print "Key Length       10 "  
-    print "AES-CMAC        " , pp(hexlify(AES_CMAC_PRF_128(unhexlify(K)[:10],unhexlify(M),10,len(unhexlify(M)))))
-    print 
+    print("AES-CMAC-PRF-128 Test Vectors")
+    print()
+    print("Example 1: len = 0")
+    pp("M               " , (K))
+    print("Key Length       18 ")  
+    pp("AES-CMAC        " , (hexlify(AES_CMAC_PRF_128(unhexlify(K),unhexlify(M),18,len(unhexlify(M))))))
+    print() 
+    print("Example 1: len = 0")
+    pp("M               " , (K))
+    print("Key Length       16 ")  
+    pp("AES-CMAC        " , (hexlify(AES_CMAC_PRF_128(unhexlify(K)[:16],unhexlify(M),16,len(unhexlify(M))))))
+    print() 
+    print("Example 1: len = 0")
+    pp("M               " , (K))
+    print("Key Length       10 ")
+    pp("AES-CMAC        " , (hexlify(AES_CMAC_PRF_128(unhexlify(K)[:10],unhexlify(M),10,len(unhexlify(M))))))
+    print() 
 
 
 
