@@ -589,7 +589,7 @@ class NDRCONSTRUCTEDTYPE(NDR):
             data = data[pad0:]
 
         if isinstance(self.fields[fieldName], NDRUniConformantArray):
-            # Array Size is at the very begining
+            # Array Size is at the very beginning
             arraySize = unpack(arrayUnPackStr, data[:arrayItemSize])[0]
         elif isinstance(self.fields[fieldName], NDRUniConformantVaryingArray):
             # NDRUniConformantVaryingArray Array
@@ -1408,7 +1408,7 @@ class NDRUNION(NDRCONSTRUCTEDTYPE):
             soFar += pad
 
         if self.structure is ():
-            return self
+            return soFar-soFar0
 
         for fieldName, fieldTypeOrClass in self.structure:
             try:
