@@ -9,7 +9,7 @@
 # Description:
 #
 # Wrapper class for SMB1/2/3 so it's transparent for the client.
-# You can still play with the low level methods (version dependant)
+# You can still play with the low level methods (version dependent)
 # by calling getSMBServer()
 #
 
@@ -28,18 +28,20 @@ class SMBConnection:
     """
     SMBConnection class
 
-    :param string remoteName: name of the remote host, can be it's NETBIOS name, IP or *\*SMBSERVER*.  If the later, and port is 139, the library will try to get the target's server name.
-    :param string remoteHost: target server's remote address (IPv4, IPv6)
+    :param string remoteName: name of the remote host, can be its NETBIOS name, IP or *\*SMBSERVER*.  If the later,
+           and port is 139, the library will try to get the target's server name.
+    :param string remoteHost: target server's remote address (IPv4, IPv6) or FQDN
     :param string/optional myName: client's NETBIOS name
     :param integer/optional sess_port: target port to connect
     :param integer/optional timeout: timeout in seconds when receiving packets
-    :param optional preferredDialect: the dialect desired to talk with the target server. If not specified the highest one available will be used
+    :param optional preferredDialect: the dialect desired to talk with the target server. If not specified the highest
+           one available will be used
     :param optional boolean manualNegotiate: the user manually performs SMB_COM_NEGOTIATE
 
     :return: a SMBConnection instance, if not raises a SessionError exception
     """
 
-    def __init__(self, remoteName='', remoteHost='', myName=None, sess_port=445, timeout=60, preferredDialect=None,
+    def __init__(self, remoteName='', remoteHost='', myName=None, sess_port=nmb.SMB_SESSION_PORT, timeout=60, preferredDialect=None,
                  existingConnection=None, manualNegotiate=False):
 
         self._SMBConnection = 0
@@ -50,11 +52,20 @@ class SMBConnection:
         self._remoteHost    = remoteHost
         self._remoteName    = remoteName
         self._timeout       = timeout
+        self._preferredDialect = preferredDialect
+        self._existingConnection = existingConnection
+        self._manualNegotiate = manualNegotiate
+        self._doKerberos = False
+        self._kdcHost = None
+        self._useCache = True
+        self._ntlmFallback = True
 
         if existingConnection is not None:
             # Existing Connection must be a smb or smb3 instance
             assert ( isinstance(existingConnection,smb.SMB) or isinstance(existingConnection, smb3.SMB3))
             self._SMBConnection = existingConnection
+            self._preferredDialect = self._SMBConnection.getDialect()
+            self._doKerberos = self._SMBConnection.getKerberos()
             return
 
         ##preferredDialect = smb.SMB_DIALECT
@@ -76,15 +87,33 @@ class SMBConnection:
 
         :return: True, raises a Session Error if error.
         """
+
+        # If port 445 and the name sent is *SMBSERVER we're setting the name to the IP. This is to help some old
+        # applications still believing
+        # *SMSBSERVER will work against modern OSes. If port is NETBIOS_SESSION_PORT the user better know about i
+        # *SMBSERVER's limitations
+        if self._sess_port == nmb.SMB_SESSION_PORT and self._remoteName == '*SMBSERVER':
+            self._remoteName = self._remoteHost
+        elif self._sess_port == nmb.NETBIOS_SESSION_PORT and self._remoteName == '*SMBSERVER':
+            # If remote name is *SMBSERVER let's try to query its name.. if can't be guessed, continue and hope for the best
+            nb = nmb.NetBIOS()
+            try:
+                res = nb.getnetbiosname(self._remoteHost)
+            except:
+                pass
+            else:
+                self._remoteName = res
+
         hostType = nmb.TYPE_SERVER
         if preferredDialect is None:
             # If no preferredDialect sent, we try the highest available one.
-            packet = self._negotiateSession(self._myName, self._remoteName, self._remoteHost, self._sess_port,
-                                            self._timeout, True, flags1=flags1, flags2=flags2, data=negoData)
+            packet = self.negotiateSessionWildcard(self._myName, self._remoteName, self._remoteHost, self._sess_port,
+                                                   self._timeout, True, flags1=flags1, flags2=flags2, data=negoData)
             if packet[0] == '\xfe':
                 # Answer is SMB2 packet
                 self._SMBConnection = smb3.SMB3(self._remoteName, self._remoteHost, self._myName, hostType,
-                                                self._sess_port, self._timeout, session=self._nmbSession)
+                                                self._sess_port, self._timeout, session=self._nmbSession,
+                                                negSessionResponse=SMB2Packet(packet))
             else:
                 # Answer is SMB packet, sticking to SMBv1
                 self._SMBConnection = smb.SMB(self._remoteName, self._remoteHost, self._myName, hostType,
@@ -98,7 +127,7 @@ class SMBConnection:
                 self._SMBConnection = smb3.SMB3(self._remoteName, self._remoteHost, self._myName, hostType,
                                                 self._sess_port, self._timeout, preferredDialect=preferredDialect)
             else:
-                LOG.critical("Unknown dialect ", preferredDialect)
+                LOG.critical("Unknown dialect %s", preferredDialect)
                 raise
 
         # propagate flags to the smb sub-object
@@ -108,8 +137,8 @@ class SMBConnection:
 
         return True
 
-    def _negotiateSession(self, myName, remoteName, remoteHost, sess_port, timeout, extended_security=True, flags1=0,
-                          flags2=0, data=None):
+    def negotiateSessionWildcard(self, myName, remoteName, remoteHost, sess_port, timeout, extended_security=True, flags1=0,
+                                 flags2=0, data=None):
         # Here we follow [MS-SMB2] negotiation handshake trying to understand what dialects
         # (including SMB1) is supported on the other end.
 
@@ -119,13 +148,6 @@ class SMBConnection:
             if i > -1:
                 myName = myName[:i]
 
-        # If port 445 and the name sent is *SMBSERVER we're setting the name to the IP. This is to help some old
-        # applications still believing
-        # *SMSBSERVER will work against modern OSes. If port is NETBIOS_SESSION_PORT the user better know about i
-        # *SMBSERVER's limitations
-        if sess_port == 445 and remoteName == '*SMBSERVER':
-            remoteName = remoteHost
-
         tries = 0
         smbp = smb.NewSMBPacket()
         smbp['Flags1'] = flags1
@@ -160,6 +182,9 @@ class SMBConnection:
         return resp.get_trailer()
 
 
+    def getNMBServer(self):
+        return self._nmbSession
+
     def getSMBServer(self):
         """
         returns the SMB/SMB3 instance being used. Useful for calling low level methods
@@ -172,9 +197,18 @@ class SMBConnection:
     def getServerName(self):
         return self._SMBConnection.get_server_name()
 
+    def getClientName(self):
+        return self._SMBConnection.get_client_name()
+
     def getRemoteHost(self):
         return self._SMBConnection.get_remote_host()
 
+    def getRemoteName(self):
+        return self._SMBConnection.get_remote_name()
+
+    def setRemoteName(self, name):
+        return self._SMBConnection.set_remote_name(name)
+
     def getServerDomain(self):
         return self._SMBConnection.get_server_domain()
 
@@ -221,13 +255,14 @@ class SMBConnection:
 
         :return: None, raises a Session Error if error.
         """
+        self._ntlmFallback = ntlmFallback
         try:
             if self.getDialect() == smb.SMB_DIALECT:
                 return self._SMBConnection.login(user, password, domain, lmhash, nthash, ntlmFallback)
             else:
                 return self._SMBConnection.login(user, password, domain, lmhash, nthash)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def kerberosLogin(self, user, password, domain='', lmhash='', nthash='', aesKey='', kdcHost=None, TGT=None,
                       TGS=None, useCache=True):
@@ -253,6 +288,9 @@ class SMBConnection:
         from impacket.krb5 import constants
         from impacket.ntlm import compute_lmhash, compute_nthash
 
+        self._kdcHost = kdcHost
+        self._useCache = useCache
+
         if TGT is not None or TGS is not None:
             useCache = False
 
@@ -263,13 +301,13 @@ class SMBConnection:
                 # No cache present
                 pass
             else:
-                # retrieve user and domain information from CCache file if needed
-                if user == '' and len(ccache.principal.components) > 0:
-                    user=ccache.principal.components[0]['data']
+                LOG.debug("Using Kerberos Cache: %s" % os.getenv('KRB5CCNAME'))
+                # retrieve domain information from CCache file if needed
                 if domain == '':
                     domain = ccache.principal.realm['data']
-                LOG.debug("Using Kerberos Cache: %s" % os.getenv('KRB5CCNAME'))
-                principal = 'cifs/%s@%s' % (self.getRemoteHost().upper(), domain.upper())
+                    LOG.debug('Domain retrieved from CCache: %s' % domain)
+
+                principal = 'cifs/%s@%s' % (self.getRemoteName().upper(), domain.upper())
                 creds = ccache.getCredential(principal)
                 if creds is None:
                     # Let's try for the TGT and go from there
@@ -281,9 +319,17 @@ class SMBConnection:
                     else:
                         LOG.debug("No valid credentials found in cache. ")
                 else:
-                    TGS = creds.toTGS()
+                    TGS = creds.toTGS(principal)
                     LOG.debug('Using TGS from cache')
 
+                # retrieve user information from CCache file if needed
+                if user == '' and creds is not None:
+                    user = creds['client'].prettyPrint().split('@')[0]
+                    LOG.debug('Username retrieved from CCache: %s' % user)
+                elif user == '' and len(ccache.principal.components) > 0:
+                    user = ccache.principal.components[0]['data']
+                    LOG.debug('Username retrieved from CCache: %s' % user)
+
         while True:
             try:
                 if self.getDialect() == smb.SMB_DIALECT:
@@ -292,7 +338,7 @@ class SMBConnection:
                 return self._SMBConnection.kerberosLogin(user, password, domain, lmhash, nthash, aesKey, kdcHost, TGT,
                                                          TGS)
             except (smb.SessionError, smb3.SessionError), e:
-                raise SessionError(e.get_error_code())
+                raise SessionError(e.get_error_code(), e.get_error_packet())
             except KerberosError, e:
                 if e.getErrorCode() == constants.ErrorCodes.KDC_ERR_ETYPE_NOSUPP.value:
                     # We might face this if the target does not support AES
@@ -312,13 +358,13 @@ class SMBConnection:
         try:
             return self._SMBConnection.isGuestSession()
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def logoff(self):
         try:
             return self._SMBConnection.logoff()
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
 
     def connectTree(self,share):
@@ -331,14 +377,14 @@ class SMBConnection:
         try:
             return self._SMBConnection.connect_tree(share)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
 
     def disconnectTree(self, treeId):
         try:
             return self._SMBConnection.disconnect_tree(treeId)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
 
     def listShares(self):
@@ -349,7 +395,7 @@ class SMBConnection:
         """
         # Get the shares through RPC
         from impacket.dcerpc.v5 import transport, srvs
-        rpctransport = transport.SMBTransport(self.getRemoteHost(), self.getRemoteHost(), filename=r'\srvsvc',
+        rpctransport = transport.SMBTransport(self.getRemoteName(), self.getRemoteHost(), filename=r'\srvsvc',
                                               smb_connection=self)
         dce = rpctransport.get_dce_rpc()
         dce.connect()
@@ -371,7 +417,7 @@ class SMBConnection:
         try:
             return self._SMBConnection.list_path(shareName, path, password)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def createFile(self, treeId, pathName, desiredAccess=GENERIC_ALL,
                    shareMode=FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
@@ -415,14 +461,14 @@ class SMBConnection:
             try:
                 return self._SMBConnection.nt_create_andx(treeId, pathName, cmd = ntCreate)
             except (smb.SessionError, smb3.SessionError), e:
-                raise SessionError(e.get_error_code())
+                raise SessionError(e.get_error_code(), e.get_error_packet())
         else:
             try:
                 return self._SMBConnection.create(treeId, pathName, desiredAccess, shareMode, creationOption,
                                                   creationDisposition, fileAttributes, impersonationLevel,
                                                   securityFlags, oplockLevel, createContexts)
             except (smb.SessionError, smb3.SessionError), e:
-                raise SessionError(e.get_error_code())
+                raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def openFile(self, treeId, pathName, desiredAccess=FILE_READ_DATA | FILE_WRITE_DATA, shareMode=FILE_SHARE_READ,
                  creationOption=FILE_NON_DIRECTORY_FILE, creationDisposition=FILE_OPEN,
@@ -465,14 +511,14 @@ class SMBConnection:
             try:
                 return self._SMBConnection.nt_create_andx(treeId, pathName, cmd = ntCreate)
             except (smb.SessionError, smb3.SessionError), e:
-                raise SessionError(e.get_error_code())
+                raise SessionError(e.get_error_code(), e.get_error_packet())
         else:
             try:
                 return self._SMBConnection.create(treeId, pathName, desiredAccess, shareMode, creationOption,
                                                   creationDisposition, fileAttributes, impersonationLevel,
                                                   securityFlags, oplockLevel, createContexts)
             except (smb.SessionError, smb3.SessionError), e:
-                raise SessionError(e.get_error_code())
+                raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def writeFile(self, treeId, fileId, data, offset=0):
         """
@@ -488,7 +534,7 @@ class SMBConnection:
         try:
             return self._SMBConnection.writeFile(treeId, fileId, data, offset)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
 
     def readFile(self, treeId, fileId, offset = 0, bytesToRead = None, singleCall = True):
@@ -519,7 +565,7 @@ class SMBConnection:
                     toRead = ''
                     break
                 else:
-                    raise SessionError(e.get_error_code())
+                    raise SessionError(e.get_error_code(), e.get_error_packet())
 
             data += bytesRead
             if len(data) >= bytesToRead:
@@ -548,7 +594,7 @@ class SMBConnection:
         try:
             return self._SMBConnection.close(treeId, fileId)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def deleteFile(self, shareName, pathName):
         """
@@ -563,7 +609,7 @@ class SMBConnection:
         try:
             return self._SMBConnection.remove(shareName, pathName)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def queryInfo(self, treeId, fileId):
         """
@@ -582,7 +628,7 @@ class SMBConnection:
                 res = self._SMBConnection.queryInfo(treeId, fileId)
             return smb.SMBQueryFileStandardInfo(res)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def createDirectory(self, shareName, pathName ):
         """
@@ -597,7 +643,7 @@ class SMBConnection:
         try:
             return self._SMBConnection.mkdir(shareName, pathName)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def deleteDirectory(self, shareName, pathName):
         """
@@ -612,7 +658,7 @@ class SMBConnection:
         try:
             return self._SMBConnection.rmdir(shareName, pathName)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def waitNamedPipe(self, treeId, pipeName, timeout = 5):
         """
@@ -628,7 +674,7 @@ class SMBConnection:
         try:
             return self._SMBConnection.waitNamedPipe(treeId, pipeName, timeout = timeout)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def transactNamedPipe(self, treeId, fileId, data, waitAnswer = True):
         """
@@ -645,7 +691,7 @@ class SMBConnection:
         try:
             return self._SMBConnection.TransactNamedPipe(treeId, fileId, data, waitAnswer = waitAnswer)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
 
     def transactNamedPipeRecv(self):
@@ -658,7 +704,7 @@ class SMBConnection:
         try:
             return self._SMBConnection.TransactNamedPipeRecv()
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def writeNamedPipe(self, treeId, fileId, data, waitAnswer = True):
         """
@@ -678,7 +724,7 @@ class SMBConnection:
             else:
                 return self.writeFile(treeId, fileId, data, 0)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
 
     def readNamedPipe(self,treeId, fileId, bytesToRead = None ):
@@ -696,7 +742,7 @@ class SMBConnection:
         try:
             return self.readFile(treeId, fileId, bytesToRead = bytesToRead, singleCall = True)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
 
     def getFile(self, shareName, pathName, callback, shareAccessMode = None):
@@ -717,7 +763,7 @@ class SMBConnection:
             else:
                 return self._SMBConnection.retr_file(shareName, pathName, callback, shareAccessMode=shareAccessMode)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def putFile(self, shareName, pathName, callback, shareAccessMode = None):
         """
@@ -737,11 +783,11 @@ class SMBConnection:
             else:
                 return self._SMBConnection.stor_file(shareName, pathName, callback, shareAccessMode)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def rename(self, shareName, oldPath, newPath):
         """
-        rename a file/directory
+        renames a file/directory
 
         :param string shareName: name for the share where the files/directories are
         :param string oldPath: the old path name or the directory/file to rename
@@ -754,13 +800,31 @@ class SMBConnection:
         try:
             return self._SMBConnection.rename(shareName, oldPath, newPath)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
+
+    def reconnect(self):
+        """
+        reconnects the SMB object based on the original options and credentials used. Only exception is that
+        manualNegotiate will not be honored.
+        Not only the connection will be created but also a login attempt using the original credentials and
+        method (Kerberos, PtH, etc)
+
+        :return: True, raises a SessionError exception if error
+        """
+        userName, password, domain, lmhash, nthash, aesKey, TGT, TGS = self.getCredentials()
+        self.negotiateSession(self._preferredDialect)
+        if self._doKerberos is True:
+            self.kerberosLogin(userName, password, domain, lmhash, nthash, aesKey, self._kdcHost, TGT, TGS, self._useCache)
+        else:
+            self.login(userName, password, domain, lmhash, nthash, self._ntlmFallback)
+
+        return True
 
     def setTimeout(self, timeout):
         try:
             return self._SMBConnection.set_timeout(timeout)
         except (smb.SessionError, smb3.SessionError), e:
-            raise SessionError(e.get_error_code())
+            raise SessionError(e.get_error_code(), e.get_error_packet())
 
     def getSessionKey(self):
         if self.getDialect() == smb.SMB_DIALECT:
@@ -773,6 +837,18 @@ class SMBConnection:
             return self._SMBConnection.set_session_key(key)
         else:
             return self._SMBConnection.setSessionKey(key)
+            
+    def close(self):
+        """
+        logs off and closes the underlying _NetBIOSSession()
+
+        :return: None
+        """
+        try:
+            self.logoff()
+        except:
+            pass
+        self._SMBConnection.close_session()
 
 class SessionError(Exception):
     """
